@@ -18,10 +18,9 @@
 #include "buffer.hpp"
 #include "os.hpp"
 
-// This is the only file allowed to include config.h because config.h is
-// only produced when building with cmake. When using the zig build system,
-// zig0.cpp is never touched.
+#ifndef ZIG_VERSION_STRING
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -46,13 +45,11 @@ static int print_full_usage(const char *arg0, FILE *file, int return_code) {
         "  -OReleaseFast                build with optimizations on and safety off\n"
         "  -OReleaseSafe                build with optimizations on and safety on\n"
         "  -OReleaseSmall               build with size optimizations on and safety off\n"
-        "  --single-threaded            source may assume it is only used single-threaded\n"
+        "  -fsingle-threaded            source may assume it is only used single-threaded\n"
         "  -dynamic                     create a shared library (.so; .dll; .dylib)\n"
         "  --strip                      exclude debug symbols\n"
         "  -target [name]               <arch>-<os>-<abi> see the targets command\n"
         "  -mcpu [cpu]                  specify target CPU and feature set\n"
-        "  --verbose-tokenize           enable compiler debug output for tokenization\n"
-        "  --verbose-ast                enable compiler debug output for AST parsing\n"
         "  --verbose-ir                 enable compiler debug output for Zig IR\n"
         "  --verbose-llvm-ir            enable compiler debug output for LLVM IR\n"
         "  --verbose-cimport            enable compiler debug output for C imports\n"
@@ -115,6 +112,8 @@ static Os get_zig_os_type(ZigLLVM_OSType os_type) {
             return OsAMDHSA;
         case ZigLLVM_PS4:
             return OsPS4;
+        case ZigLLVM_PS5:
+            return OsPS5;
         case ZigLLVM_ELFIAMCU:
             return OsELFIAMCU;
         case ZigLLVM_TvOS:
@@ -135,6 +134,10 @@ static Os get_zig_os_type(ZigLLVM_OSType os_type) {
             return OsWASI;
         case ZigLLVM_Emscripten:
             return OsEmscripten;
+        case ZigLLVM_DriverKit:
+            return OsDriverKit;
+        case ZigLLVM_ShaderModel:
+            return OsShaderModel;
     }
     zig_unreachable();
 }
@@ -160,6 +163,17 @@ static void get_native_target(ZigTarget *target) {
     }
 }
 
+static const char* get_baseline_llvm_cpu_name(ZigLLVM_ArchType arch) {
+    return "";
+}
+
+static const char* get_baseline_llvm_cpu_features(ZigLLVM_ArchType arch) {
+    switch (arch) {
+        case ZigLLVM_riscv64: return "+a,+c,+d,+m";
+        default: return "";
+    }
+}
+
 static Error target_parse_triple(struct ZigTarget *target, const char *zig_triple, const char *mcpu,
         const char *dynamic_linker)
 {
@@ -178,8 +192,8 @@ static Error target_parse_triple(struct ZigTarget *target, const char *zig_tripl
         } else if (strcmp(mcpu, "baseline") == 0) {
             target->is_native_os = false;
             target->is_native_cpu = false;
-            target->llvm_cpu_name = "";
-            target->llvm_cpu_features = "";
+            target->llvm_cpu_name = get_baseline_llvm_cpu_name(target->arch);
+            target->llvm_cpu_features = get_baseline_llvm_cpu_features(target->arch);
         } else {
             const char *msg = "stage0 can't handle CPU/features in the target";
             stage2_panic(msg, strlen(msg));
@@ -220,6 +234,9 @@ static Error target_parse_triple(struct ZigTarget *target, const char *zig_tripl
             const char *msg = "stage0 can't handle CPU/features in the target";
             stage2_panic(msg, strlen(msg));
         }
+
+        target->llvm_cpu_name = get_baseline_llvm_cpu_name(target->arch);
+        target->llvm_cpu_features = get_baseline_llvm_cpu_features(target->arch);
     }
 
     return ErrorNone;
@@ -285,7 +302,7 @@ int main(int argc, char **argv) {
                 optimize_mode = BuildModeSafeRelease;
             } else if (strcmp(arg, "-OReleaseSmall") == 0) {
                 optimize_mode = BuildModeSmallRelease;
-            } else if (strcmp(arg, "--single-threaded") == 0) {
+            } else if (strcmp(arg, "-fsingle-threaded") == 0) {
                 single_threaded = true;
             } else if (strcmp(arg, "--help") == 0) {
                 return print_full_usage(arg0, stdout, EXIT_SUCCESS);
